package hep.wired.corbaheprep;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.*;
import javax.swing.filechooser.FileFilter;
import org.omg.CORBA.ORB;

import hep.wired.hepeventserver.idl.HepEventServer;
import hep.wired.hepeventserver.idl.HepEventServerHelper;
import java.net.InetAddress;
import java.net.UnknownHostException;
import org.freehep.application.ApplicationEvent;
import org.freehep.application.ApplicationListener;
import org.freehep.application.studio.Plugin;
import org.freehep.application.studio.Studio;
import org.freehep.jas.services.DataSource;
import org.freehep.jas.services.FileHandler;
import org.freehep.record.source.RecordSource;
import org.freehep.swing.ExtensionFileFilter;
import org.freehep.swing.wizard.WizardPage;
import org.freehep.util.FreeHEPLookup;

/**
 * A JAS3 plugin for Procman and HepEventServer.
 *
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: CORBAHepRepPlugin.java 14058 2012-11-02 23:37:21Z onoprien $
 */
public class CORBAHepRepPlugin extends Plugin implements DataSource, FileHandler, CORBASourceFactory {

// -- Private parts : ----------------------------------------------------------
// -- Plugin initialization : --------------------------------------------------
  protected void init() throws IOException {
    getApplication().getLookup().add(this);
  }

// -- Implementing DataSource : ------------------------------------------------
  public String getName() {
    return "HepRep CORBA Servers";
  }

  public WizardPage getWizardPage() {
    return new CORBAServerWizardPage();
  }

// -- Implementing FileHandler : -----------------------------------------------
  public boolean accept(File file) throws IOException {
    return file.getName().endsWith(".ior");
  }

  public FileFilter getFileFilter() {
    return new ExtensionFileFilter(".ior", "CORBA IOR File");
  }

  public void openFile(File file) throws IOException {
    String id = null;
    BufferedReader in = null;
    try {
      in = new BufferedReader(new FileReader(file));
      id = in.readLine();
    } catch (Exception x) {
      throw new IOException("Failed to read from " + file, x);
    } finally {
      if (in != null) {
        in.close();
      }
    }
    ORB orb = ORB.init(new String[0], System.getProperties());
    org.omg.CORBA.Object corbaObject = orb.string_to_object(id);
    HepEventServer hepEventServer = HepEventServerHelper.narrow(corbaObject);
    Properties serverInfo = attach(hepEventServer);
    CORBASourceFactory factory = getRecordSourceFactory(serverInfo.getProperty("servername"));
    final RecordSource source = factory.createRecordSource(file.getName(), hepEventServer, serverInfo);
    Studio studio = getApplication();
    studio.getLookup().add(source);
    studio.addApplicationListener(new ApplicationListener() {
      public void initializationComplete(ApplicationEvent e) {
      }

      public void aboutToExit(ApplicationEvent event) {
        try {
          source.close();
        } catch (Exception e) {
          System.err.println("Source '" + getName() + "' was not properly closed: " + e);
        }
      }
    });
  }

// -- Attaching to CORBA server : ----------------------------------------------
  /**
   * Attach the specified CORBA server.
   */
  static public Properties attach(HepEventServer server) throws IOException {

    String userName = System.getProperty("user.name");
    String hostName = null;
    try {
      InetAddress inet = InetAddress.getLocalHost();
      hostName = inet.getCanonicalHostName();
    } catch (UnknownHostException uhe) {
      hostName = "wired";
    }

    String user = userName + "@" + hostName;
    String client = "client=WIRED4;version=4.1;user=" + user + ";";
    String response;
    try {
      response = server.attach(client);
    } catch (Exception x) {
      throw new IOException("Failed to attach CORBA server", x);
    }

    String[] keyValues = response.split(";");
    Properties serverInfo = new Properties();
    for (int k = 0; k < keyValues.length; k++) {
      String[] keyValue = keyValues[k].split("=", 2);
      if (keyValue.length != 2) {
        throw new IOException("Illegal response from server: '" + keyValues[k] + "'");
      }
      // special case to extract server _name "SomeServerName version=1.0;xxx=yyy"
      if (k == 0) {
        int space = keyValue[0].lastIndexOf(' ');
        if (space >= 0) {
          serverInfo.setProperty("servername", keyValue[0].substring(0, space));
          keyValue[0] = keyValue[0].substring(space + 1);
        }
      }
      serverInfo.setProperty(keyValue[0].toLowerCase(), keyValue[1]);
    }

    return serverInfo;
  }

// -- Implementing CORBASourceFactory : ----------------------------------------
  /**
   * Creates and returns a generic implementations of {@link CORBARecordSource}.
   */
  public CORBARecordSource createRecordSource(String name, HepEventServer server, Properties serverInfo) {
    return new CORBARecordSource(name, server, serverInfo);
  }

// -- Handling CORBASourceFactory lookup : -------------------------------------
  public CORBASourceFactory getRecordSourceFactory(String serverName) {
    FreeHEPLookup lookup = getApplication().getLookup();
    CORBASourceFactory factory = (CORBASourceFactory) lookup.lookupItem(new FreeHEPLookup.Template(CORBASourceFactory.class, serverName, null)).getInstance();
    return factory == null ? this : factory;
  }

// -- Overriding Object : ------------------------------------------------------
  public String toString() {
    return getName();
  }
}
