package hep.wired.corbaheprep;

import hep.wired.hepeventserver.idl.HepEventServer;
import hep.wired.hepeventserver.idl.HepEventServerHelper;
import hep.wired.jprocman.Goal;
import hep.wired.jprocman.GoalSet;
import hep.wired.jprocman.ProcManClient;
import hep.wired.jprocman.ProcProxy;
import hep.wired.jprocman.corba.idl.ProcManServer;
import hep.wired.jprocman.corba.idl.ProcManServerHelper;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreeSelectionModel;

import org.freehep.application.Application;
import org.freehep.application.studio.Studio;
import org.freehep.swing.ExtensionFileFilter;
import org.freehep.swing.layout.TableLayout;
import org.freehep.swing.wizard.Finishable;
import org.freehep.swing.wizard.HasNextPages;
import org.freehep.swing.wizard.WizardPage;
import org.omg.CORBA.ORB;
import org.omg.CORBA.SystemException;

/**
 * @author Mark Donszelmann
 * @version $Id: CORBAServerWizardPage.java 14056 2012-10-31 23:20:40Z onoprien $
 */
public class CORBAServerWizardPage extends WizardPage implements Finishable, HasNextPages {

    private ORB orb;

    private JTree tree;

    private TreeNode selectedNode;

    private JLabel descriptionLabel;

    private JTextArea iorLabel;

    private JButton iorBrowse;

    private JTextField iorName;

    private JButton iorButton;

    private HepEventServerWizardPage hepEventServerPage;

    private RootTreeNode root;

    private DefaultTreeModel model;

    public CORBAServerWizardPage() {
        selectedNode = null;
        root = null;
        model = null;

        hepEventServerPage = new HepEventServerWizardPage();

        try {
            tree = new JTree();
            tree.setRootVisible(false);
            tree.setShowsRootHandles(true);
            tree.getSelectionModel().setSelectionMode(
                    TreeSelectionModel.SINGLE_TREE_SELECTION);
            tree.addTreeSelectionListener(new TreeSelectionListener() {
                public void valueChanged(TreeSelectionEvent e) {
                    selectedNode = (TreeNode) tree
                            .getLastSelectedPathComponent();
                    CORBATreeNode node = (CORBATreeNode) selectedNode;
                    if (node != null) {
                        descriptionLabel.setText(node.getDescription());
                        iorLabel.setText(node.getID().toString());
                        iorButton.setEnabled(true);
                        doEnable();
                    }
                }
            });
            tree.setCellRenderer(new DefaultTreeCellRenderer() {
                public Component getTreeCellRendererComponent(JTree tree,
                        Object value, boolean sel, boolean expanded,
                        boolean leaf, int row, boolean hasFocus) {
                    Component c = super.getTreeCellRendererComponent(tree,
                            value, sel, expanded, leaf, row, hasFocus);
                    Font f = c.getFont();
                    if (value instanceof FailedTreeNode) {
                        c.setFont(f.deriveFont(Font.ITALIC));
                    } else {
                        c.setFont(f.deriveFont(Font.PLAIN));
                    }
                    return c;
                }
            });

            setLayout(new BorderLayout());

            JPanel box = new JPanel();
            box.setLayout(new TableLayout());
            add(box, BorderLayout.CENTER);

            JScrollPane treeBox = new JScrollPane(tree);
            treeBox.setBorder(new TitledBorder(new EtchedBorder(),
                    "CORBA Server:"));
            box.add("0 * [5 5 5 5] wh", treeBox);

            descriptionLabel = new JLabel();
            descriptionLabel.setBorder(new TitledBorder(new EtchedBorder(),
                    "Description:"));
            box.add("0 * [5 5 5 5] w", descriptionLabel);

            JPanel iorBox = new JPanel();
            iorBox.setBorder(new TitledBorder(new EtchedBorder(),
                    "IOR, URL or File:"));
            iorBox.setLayout(new TableLayout());
            box.add("0 * [5 5 5 5] w", iorBox);

            iorLabel = new JTextArea(1, 72);
            iorLabel.setFont(box.getFont());
            iorLabel.setBorder(new BevelBorder(BevelBorder.LOWERED));
            iorLabel.setLineWrap(true);
            iorLabel.addCaretListener(new ButtonEnabler());
            iorBox.add("0 0 [3 3 3 3] wh", iorLabel);

            iorBrowse = new JButton("Browse...");
            iorBrowse.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    String s = chomp(iorLabel.getText().trim());
                    if (isCorbaURL(s))
                        s = null;
                    JFileChooser chooser = new JFileChooser(s);
                    chooser.setFileFilter(new ExtensionFileFilter(".ior",
                            "CORBA IOR File"));
                    int result = chooser.showOpenDialog(iorBrowse);
                    if (result == JFileChooser.APPROVE_OPTION) {
                        iorLabel.setText(chooser.getSelectedFile().getPath());
                    }
                }
            });
            iorBox.add("1 0 [3 3 3 3] wr", iorBrowse);

            iorName = new JTextField("New Node");
            iorName.addCaretListener(new ButtonEnabler());
            iorBox.add("0 1 [3 3 3 3] lw", iorName);

            iorButton = new JButton("Add to List");
            iorButton.setEnabled(false);
            iorButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    String s = chomp(iorLabel.getText().trim());

                    boolean result = root.add(iorName.getText(), s);
                    if (result) {
                        int index = root.getChildCount() - 1;
                        model.nodesWereInserted(root, new int[] { index });
                        tree.setSelectionInterval(index, index);
                    } else {
                        JOptionPane.showMessageDialog(
                                CORBAServerWizardPage.this,
                                "Could not add new node: " + s, "Error",
                                JOptionPane.ERROR_MESSAGE);
                    }
                }
            });
            iorBox.add("1 1 [3 3 3 3] r", iorButton);

        } catch (SystemException e) {
            JOptionPane.showMessageDialog(this, "Exception happened: " + e,
                    "Error", JOptionPane.ERROR_MESSAGE);
        }
    }

    // FIXME: maybe move to some String class in FreeHEP
    private String chomp(String s) {
        if (s.length() == 0)
            return s;

        int lastIndex = s.length() - 1;
        char last = s.charAt(lastIndex);
        if (last == '\n') {
            if ((lastIndex > 0) && (s.charAt(lastIndex - 1) == '\r')) {
                lastIndex--;
            }
        } else if (last != '\r') {
            lastIndex++;
        }
        return s.substring(0, lastIndex);
    }

    protected void beforeShowing() {
        if (orb != null)
            return;
        orb = ORB.init(new String[0], System.getProperties());
        if (root == null) {
            root = new RootTreeNode();
            root.add("HepRep CORBA Servers at SLAC",
                    "http://www.slac.stanford.edu/~wiredces/prodservers.ior");
            // root.add("IR2 HepRep CORBA Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/ir2prodservers.ior");
            // root.add("ROB HepRep CORBA Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/robprodservers.ior");
            // root.add("Test Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/test.ior");
            // root.add("OTest Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/otest.ior");
            // root.add("MC Truth Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/mctruth.ior");
            // root.add("List Test Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/listtest.ior");
            // root.add("Info Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/info.ior");
            // root.add("CM2 Test Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/cm2test.ior");
            // root.add("Procman Servers at SLAC",
            // "http://www.slac.stanford.edu/~wiredces/procmanserver.ior");
            model = new DefaultTreeModel(root, true);
            tree.setModel(model);
        }

        // make frame resizeable, FIXME, maybe this should be the default...
        Component c = SwingUtilities.getRoot(this);
        if (c instanceof Frame)
            ((Frame) c).setResizable(true);
        if (c instanceof Dialog)
            ((Dialog) c).setResizable(true);
    }

    protected boolean getNextEnabled() {
        return (selectedNode != null)
                && (selectedNode instanceof HepEventServerTreeNode);
    }

    public WizardPage[] getNextWizardPages() {
        return new WizardPage[] { hepEventServerPage };
    }

    public WizardPage getNext() {
        try {
            hepEventServerPage.setHepEventServer(
                    ((HepEventServerTreeNode) selectedNode).getName(),
                    connect());
        } catch (IOException e) {
            // FIXME handle
            e.printStackTrace();
        }
        // FIXME what about CORBA Exceptions
        return hepEventServerPage;
    }

    protected boolean getFinishEnabled() {
        return selectedNode instanceof HepEventServerTreeNode;
    }

    private HepEventServer connect() throws IOException {
        HepEventServerTreeNode node = (HepEventServerTreeNode) selectedNode;
        org.omg.CORBA.Object corbaObject = orb.string_to_object((String) node.getID());
        return HepEventServerHelper.narrow(corbaObject);
    }

    public void onFinish() {
        try {
            HepEventServer server = connect();
            CORBARecordSource source = new CORBARecordSource(((HepEventServerTreeNode) selectedNode).getName(), server, CORBAHepRepPlugin.attach(server));
            ((Studio)Application.getApplication()).getLookup().add(source);
        } catch (IOException e) {
            // FIXME handle
            e.printStackTrace();
        }
        // FIXME what about CORBA Exceptions

        dispose();
    }

    public void onWasFinished() {
    }

    public void onCancel() {
        if (orb != null) {
            try {
                orb.shutdown(false);
                orb.destroy();
            } catch (SystemException e) {
                System.out.println("ORB Shutdown failed: " + e);
            }
        }
    }

    // NOTE: these URL's are handled by Sun's implementation of CORBA in JDK 1.5
    // see com.sun.corba.se.impl.resolver.INSURLOperationImpl
    // see com.sun.corba.se.impl.naming.namingutil.INSURLHandler
    private boolean isCorbaURL(String url) {
        return url.startsWith("IOR:") || // JDK 1.4
                url.startsWith("corbaloc:") || // JDK 1.5
                url.startsWith("corbaname:"); // JDK 1.5
    }

    class ButtonEnabler implements CaretListener {
        public void caretUpdate(CaretEvent e) {
            String text = chomp(iorLabel.getText().trim());
            String name = chomp(iorName.getText().trim());
            iorButton.setEnabled((e.getDot() > 0)
                    || (!text.equals("") && !name.equals("")));
        }
    }

    abstract class CORBATreeNode implements TreeNode {
        public abstract String getDescription();

        public abstract Object getID();
    }

    class RootTreeNode implements TreeNode {
        private List/* <String> */names;

        private List/* <URL/IORString> */ids;

        private List/* <Object> */objects;

        public RootTreeNode() {
            names = new ArrayList();
            ids = new ArrayList();
            objects = new ArrayList();
        }

        public boolean add(String name, String id) {
            if (isCorbaURL(id)) {
                try {
                    org.omg.CORBA.Object corbaObject = orb.string_to_object(id);
                    try {
                        ProcManServer procman = ProcManServerHelper
                                .narrow(corbaObject);
                        names.add(name);
                        ids.add(id);
                        objects.add(procman);
                        return true;
                    } catch (SystemException e) {
                    }

                    try {
                        HepEventServer hepEventServer = HepEventServerHelper
                                .narrow(corbaObject);
                        names.add(name);
                        ids.add(id);
                        objects.add(hepEventServer);
                        return true;
                    } catch (SystemException e) {
                    }
                } catch (SystemException e) {
                }
                return false;
            }

            URL url = null;
            try {
                url = new URL(id);
            } catch (MalformedURLException ignored) {
                try {
                    url = new URL("file:///" + id);
                } catch (MalformedURLException e) {
                    return false;
                }
            }
            names.add(name);
            ids.add(url);
            objects.add(url);
            return true;
        }

        public Enumeration children() {
            return new Enumeration() {
                private int index = 0;

                public boolean hasMoreElements() {
                    return index < names.size();
                }

                public Object nextElement() {
                    return getChildAt(index++);
                }
            };
        }

        public boolean getAllowsChildren() {
            return true;
        }

        public TreeNode getChildAt(int index) {
            Object o = objects.get(index);
            if (o instanceof URL) {
                return new WebIORTreeNode(this, (String) names.get(index),
                        (URL) ids.get(index));
            } else if (o instanceof ProcManServer) {
                return new ProcmanTreeNode(this, (String) ids.get(index));
            } else if (o instanceof HepEventServer) {
                return new HepEventServerTreeNode(this, (String) names
                        .get(index), "No Description", (String) ids.get(index));
            }
            return new FailedTreeNode(this, (String) names.get(index),
                    "No Description", (String) ids.get(index));
        }

        public int getChildCount() {
            return names.size();
        }

        public int getIndex(TreeNode node) {
            return ids.indexOf(((CORBATreeNode) node).getID());
        }

        public TreeNode getParent() {
            return null;
        }

        public boolean isLeaf() {
            return false;
        }

        public String toString() {
            return "Root";
        }
    }

    class WebIORTreeNode extends CORBATreeNode {
        private TreeNode parent;

        private String name;

        private URL url;

        private List/* <String> */iorStrings;

        private boolean disabled;

        public WebIORTreeNode(TreeNode parent, String name, URL url) {
            super();
            this.parent = parent;
            this.name = name;
            this.url = url;
            iorStrings = new ArrayList();
            try {
                BufferedReader in = new BufferedReader(new InputStreamReader(
                        url.openStream()));
                String line;
                while ((line = in.readLine()) != null) {
                    iorStrings.add(line);
                }
                in.close();
                disabled = false;
            } catch (IOException e) {
                disabled = true;
            }
        }

        public String getDescription() {
            return "No Description";
        }

        public Object getID() {
            return url;
        }

        public Enumeration children() {
            return new Enumeration() {
                private int index = 0;

                public boolean hasMoreElements() {
                    return index < iorStrings.size();
                }

                public Object nextElement() {
                    return getChildAt(index++);
                }
            };
        }

        public boolean getAllowsChildren() {
            return !disabled;
        }

        public TreeNode getChildAt(int index) {
            String iorString = (String) iorStrings.get(index);

            if (isCorbaURL(iorString)) {
                try {
                    org.omg.CORBA.Object corbaObject = orb
                            .string_to_object(iorString);
                    try {
                        /* ProcManServer procman = */ProcManServerHelper
                                .narrow(corbaObject);
                        return new ProcmanTreeNode(this, iorString);
                    } catch (SystemException e) {
                    }

                    try {
                        /* HepEventServer hepEventServer = */HepEventServerHelper
                                .narrow(corbaObject);
                        return new HepEventServerTreeNode(this, name,
                                "No Description", iorString);
                    } catch (SystemException e) {
                    }
                } catch (SystemException se) {
                }
            }
            return new FailedTreeNode(this, name, "No Description", iorString);
        }

        public int getChildCount() {
            return disabled ? 0 : iorStrings.size();
        }

        public int getIndex(TreeNode node) {
            return iorStrings.indexOf(((CORBATreeNode) node).getID());
        }

        public TreeNode getParent() {
            return parent;
        }

        public boolean isLeaf() {
            return disabled;
        }

        public String toString() {
            return name;
        }
    }

    class ProcmanTreeNode extends CORBATreeNode {

        private TreeNode parent;

        private String iorString;

        private String name;

        private String description;

        private List/* <String> */names;

        private List/* <String> */descriptions;

        private List/* <String> */iorStrings;

        private boolean disabled;

        public ProcmanTreeNode(TreeNode parent, String iorString) {
            super();
            this.parent = parent;
            this.iorString = iorString;
            this.name = "No Name";
            this.description = "No Description";

            try {
                ProcManClient procManClient = new ProcManClient(iorString);
                GoalSet goalSet = procManClient.getGoalSetWithProcs();
                name = goalSet.getName();
                description = goalSet.getDesc();

                // add only which are not down, ready and not undefined.
                names = new ArrayList();
                descriptions = new ArrayList();
                iorStrings = new ArrayList();
                for (Iterator i = goalSet.getGoals().iterator(); i.hasNext();) {
                    Goal goal = (Goal) i.next();
                    if (goal.isDown()) {
                        names.add(goal.getName());
                        descriptions.add(goal.getDesc());
                        iorStrings.add("DOWN: " + goal.getOutageReason());
                    } else {
                        for (Iterator j = goal.getProcProxies().iterator(); j
                                .hasNext();) {
                            ProcProxy proxy = (ProcProxy) j.next();
                            if (proxy.getStatus().equals("ready")) {
                                String ior = proxy.getIOR();
                                if (ior.startsWith("IOR:")) {
                                    names.add(goal.getName());
                                    descriptions.add(goal.getDesc());
                                    iorStrings.add(ior);
                                    break;
                                }
                            }
                        }
                    }
                }
                disabled = false;
            } catch (SystemException e) {
                // FIXME, maybe we should have an error label...
                description = e.toString();
                disabled = true;
            }
        }

        public String getDescription() {
            return description;
        }

        public Object getID() {
            return iorString;
        }

        public Enumeration children() {
            return new Enumeration() {
                int index = 0;

                public boolean hasMoreElements() {
                    return index < iorStrings.size();
                }

                public Object nextElement() {
                    return getChildAt(index++);
                }
            };
        }

        public boolean getAllowsChildren() {
            return !disabled;
        }

        public TreeNode getChildAt(int index) {
            String iorString = (String) iorStrings.get(index);

            if (isCorbaURL(iorString)) {
                try {
                    org.omg.CORBA.Object corbaObject = orb
                            .string_to_object(iorString);
                    try {
                        /* ProcManServer procman = */ProcManServerHelper
                                .narrow(corbaObject);
                        return new ProcmanTreeNode(this, iorString);
                    } catch (SystemException e) {
                    }

                    try {
                        /* HepEventServer hepEventServer = */HepEventServerHelper
                                .narrow(corbaObject);
                        return new HepEventServerTreeNode(this, (String) names
                                .get(index), (String) descriptions.get(index),
                                iorString);
                    } catch (SystemException e) {
                    }
                } catch (SystemException se) {
                }
            }
            return new FailedTreeNode(this, (String) names.get(index),
                    (String) descriptions.get(index), iorString);
        }

        public int getChildCount() {
            return disabled ? 0 : iorStrings.size();
        }

        public int getIndex(TreeNode node) {
            return iorStrings.indexOf(((CORBATreeNode) node).getID());
        }

        public TreeNode getParent() {
            return parent;
        }

        public boolean isLeaf() {
            return disabled;
        }

        public String toString() {
            return name;
        }
    }

    class HepEventServerTreeNode extends CORBATreeNode {

        private TreeNode parent;

        private String name;

        private String description;

        private String iorString;

        public HepEventServerTreeNode(TreeNode parent, String name,
                String description, String iorString) {
            super();
            this.parent = parent;
            this.name = name;
            this.description = description;
            this.iorString = iorString;
        }

        public String getDescription() {
            return description;
        }

        public Object getID() {
            return iorString;
        }

        public String getName() {
            return name;
        }

        public Enumeration children() {
            return null;
        }

        public boolean getAllowsChildren() {
            return false;
        }

        public TreeNode getChildAt(int index) {
            return null;
        }

        public int getChildCount() {
            return 0;
        }

        public int getIndex(TreeNode node) {
            return -1;
        }

        public TreeNode getParent() {
            return parent;
        }

        public boolean isLeaf() {
            return true;
        }

        public String toString() {
            return name;
        }
    }

    class FailedTreeNode extends CORBATreeNode {

        private TreeNode parent;

        private String name;

        private String description;

        private String iorString;

        public FailedTreeNode(TreeNode parent, String name, String description,
                String iorString) {
            super();
            this.parent = parent;
            this.name = name;
            this.description = description;
            this.iorString = iorString;
        }

        public String getDescription() {
            return description;
        }

        public Object getID() {
            return iorString;
        }

        public String getName() {
            return name;
        }

        public Enumeration children() {
            return null;
        }

        public boolean getAllowsChildren() {
            return false;
        }

        public TreeNode getChildAt(int index) {
            return null;
        }

        public int getChildCount() {
            return 0;
        }

        public int getIndex(TreeNode node) {
            return -1;
        }

        public TreeNode getParent() {
            return parent;
        }

        public boolean isLeaf() {
            return true;
        }

        public String toString() {
            return name;
        }
    }

}
