package hep.wired.corbaheprep;

import hep.wired.hepeventserver.idl.HepEventServer;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import org.freehep.application.Application;
import org.freehep.application.studio.Studio;
import org.freehep.record.source.TagListRecordSource;
import org.freehep.swing.layout.TableLayout;
import org.freehep.swing.wizard.Finishable;
import org.freehep.swing.wizard.WizardPage;

/**
 * FIXME handle CORBA exceptions
 * 
 * @author Mark Donszelmann
 * @version $Id: HepEventServerWizardPage.java 14056 2012-10-31 23:20:40Z onoprien $
 */
public class HepEventServerWizardPage extends WizardPage implements Finishable {

    private JRadioButton listButton;

    private JTextArea list;

    private JRadioButton fileButton;

    private JTextField file;

    private JFileChooser chooser;

    private JRadioButton eventButton;

    private JTextField event;

    private boolean canFinish;

    private CORBARecordSource source;

    public HepEventServerWizardPage() {
        canFinish = false;

        setLayout(new BorderLayout());

        ButtonGroup group = new ButtonGroup();

        JPanel box = new JPanel();
        box.setLayout(new TableLayout());
        add(box, BorderLayout.CENTER);

        JPanel listBox = new JPanel();
        listBox.setLayout(new TableLayout());
        listBox.setBorder(new EtchedBorder());
        box.add("0 * [5 5 5 5] wh", listBox);

        listButton = new JRadioButton("Event List");
        listBox.add("0 * * 1 [3 3 3 3] lw", listButton);
        group.add(listButton);

        list = new JTextArea();
        listBox.add("0 * [3 3 3 3] wh", new JScrollPane(list));

        JPanel fileBox = new JPanel();
        fileBox.setLayout(new TableLayout());
        fileBox.setBorder(new EtchedBorder());
        box.add("0 * [5 5 5 5] w", fileBox);

        fileButton = new JRadioButton("Event List File");
        fileBox.add("0 * [3 3 3 3] lw", fileButton);
        group.add(fileButton);

        file = new JTextField();
        fileBox.add("0 1 [3 3 3 3] lw", file);

        final JButton browse = new JButton("Browse...");
        fileBox.add("1 1 [3 3 3 3] r", browse);
        browse.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!fileButton.isSelected()) {
                    fileButton.setSelected(true);
                    file.requestFocusInWindow();
                }

                if (chooser == null)
                    chooser = new JFileChooser();
                // if (f != null) dlg.setSelectedFile(new File(f));
                if (chooser.showDialog(HepEventServerWizardPage.this, "Select") == JFileChooser.APPROVE_OPTION) {
                    file.setText(chooser.getSelectedFile().getAbsolutePath());
                }
            }
        });

        JPanel eventBox = new JPanel();
        eventBox.setLayout(new TableLayout());
        eventBox.setBorder(new EtchedBorder());
        box.add("0 * [5 5 5 5] w", eventBox);

        eventButton = new JRadioButton("Event");
        eventBox.add("0 * [3 3 3 3] lw", eventButton);
        group.add(eventButton);

        event = new JTextField("Unselected");
        // event.setBorder(new TitledBorder(new EtchedBorder(), "Event:"));
        eventBox.add("0 * [3 3 3 3] w", event);

        // hook the components up...
        listButton.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    list.setEnabled(true);
                } else {
                    list.setEnabled(false);
                }
            }
        });

        list.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (!listButton.isSelected()) {
                    listButton.setSelected(true);
                    list.requestFocusInWindow();
                }
            }
        });

        list.addCaretListener(new CaretListener() {
            public void caretUpdate(CaretEvent e) {
                canFinish = (e.getDot() > 0) || !list.getText().equals("");
                doEnable();
            }
        });

        fileButton.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    file.setEnabled(true);
                } else {
                    file.setEnabled(false);
                }
            }
        });

        file.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (!fileButton.isSelected()) {
                    fileButton.setSelected(true);
                    file.requestFocusInWindow();
                }
            }
        });

        file.addCaretListener(new CaretListener() {
            public void caretUpdate(CaretEvent e) {
                canFinish = (e.getDot() > 0) || !file.getText().equals("");
                doEnable();
            }
        });

        eventButton.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    event.setEnabled(true);
                } else {
                    event.setEnabled(false);
                }
            }
        });

        event.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (!eventButton.isSelected()) {
                    eventButton.setSelected(true);
                    event.requestFocusInWindow();
                }
            }
        });

        event.addCaretListener(new CaretListener() {
            public void caretUpdate(CaretEvent e) {
                canFinish = (e.getDot() > 0) || !event.getText().equals("");
                doEnable();
            }
        });

        // set initial state
        list.setEnabled(false);
        file.setEnabled(false);
        eventButton.setSelected(true);
    }

    public void setHepEventServer(String name, HepEventServer server)
            throws IOException {
        source = new CORBARecordSource(name, server, CORBAHepRepPlugin.attach(server));
        event.setText("None");
    }

    protected void beforeShowing() {
    }

    protected boolean getFinishEnabled() {
        return canFinish;
    }

    public void onPrevious() {
        try {
            source.close();
        } catch (Exception e) {
            System.err.println("HepEventServerWizardPage '"
                    + source.getName() + "' was not properly closed: "
                    + e);
        }
    }

    public void onFinish() {
      List<String> eventList = null;
      if (listButton.isSelected()) {
        try {
          eventList = eventList(new StringReader(list.getText()));
        } catch (IOException e) {
          JOptionPane.showMessageDialog(this, "Cannot parse event list: "
                  + list.getText(), "Error", JOptionPane.ERROR_MESSAGE);
          return;
        }
      } else if (fileButton.isSelected()) {
        try {
          eventList = eventList(new FileReader(file.getText()));
        } catch (IOException e) {
          JOptionPane.showMessageDialog(this, "Cannot find event list: "
                  + file.getText(), "Error", JOptionPane.ERROR_MESSAGE);
          return;
        }
      } else if (eventButton.isSelected()) {
        eventList = Collections.singletonList(event.getText());
      }
      ((Studio)Application.getApplication()).getLookup().add(new TagListRecordSource(source, eventList, false));
      dispose();
    }

    public void onWasFinished() {
    }

    public void onCancel() {
    }

    private List<String> eventList(Reader reader) throws IOException {
        List<String> eventList = new ArrayList();
        BufferedReader br = new BufferedReader(reader);
        String line = br.readLine();
        while (line != null) {
            // ignore lines starting with # in the very first column
            line = line.trim();
            if ((line.length() > 0) && (line.charAt(0) != '#')) {
                eventList.add(line);
            }
            line = br.readLine();
        }
        br.close();
        return eventList;
    }
}
